#ifndef INCLUDES_TALLY_H
#define INCLUDES_TALLY_H

/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@strudl.org                *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/

#include "state_utils.h"
#include "ptr_set.h"
#include "canonical_string.h"

/*!\ingroup includes_tally_component includes_tally_interface
 *\file includes_tally.h
 * This file provides the Includes Tally component interface
*/

/*!\addtogroup includes_tally_interface*/
/*@{*/


/*!	Tally an occurrence of an hash-include argument.
	\param	arg The start of the hash-include argument.
	\param	arglen The length of the hash-include argumentument.
	\param	sym_val Where the hash-include argument is a defined symmbol,
		\e sym_def contains the canonicalised final value of
		that symbol, else NULL.

	If no hash-include argument matching \e arg exists in in the includes
	table, then a new hash-include argument is added to the table. If a
	matching hash-include argument is already in the table then that
	hash-include argument is given the definition \e sym_val if \e sym_val
	is not NULL and if the existing hash-include argument does not already
	have that definition. If \e sym_val is NULL then the matching hash-include
	argument is made undefined if it currently has a definition.

	The occurrence of the hash-include argument is reported on stdout if the
	same hash-include argument has not already been seen or if \c --once is
	not in effect. If the hash-include argument is reported and \e sym_val
	is not NULL	then the hash-include argument will be reported as a symbol
	that resolves \e sym_val.
*/
extern void
hash_include_tally(char const *arg, size_t arglen,
                   canonical_string_const_h sym_val);


/*@}*/

/*!\ingroup includes_tally_interface_state_utils*/
/*@{*/
/*! The public state of the Includes Tally component*/
PUBLIC_STATE_DEF(includes_tally)
{
    ptr_set_h includes_tab;
    /*!< Handle to the includes table*/
}
PUBLIC_STATE_T(includes_tally);
/*@}*/

/*!\addtogroup includes_tally_interface_state_utils*/
/*@{*/
IMPORT(includes_tally);
/*@}*/


#endif /* EOF*/
