#ifndef LINE_SELECT_H
#define LINE_SELECT_H

/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/

#include "state_utils.h"

/*!\ingroup line_select_component line_select_interface
 *\file line_select.h
 * This file provides the Line Select component interface
*/

/*!\addtogroup line_select_interface*/
/*@{*/

/*! Say whether the current input line is eligible for evaluation by a
	specified command, given the other options in effecy.
	\param	cmd_code	The command code of the command in question.
	\param	directive	The preprocessor directive found on the
		current line.
*/
extern bool
eligible_line(int cmd_code, int directive);

/*! Say whether we are dropping the current line,

	If \c --complement is not in effect then the function
	returns the same value as \e dead_line(). Otherwise the
	opposite value.
*/
extern bool
dropping_line(void);

/*@}*/

/*!\ingroup line_select_interface_state_utils*/
/*@{*/

/*! The public state of the Line Select component
*/
PUBLIC_STATE_DEF(line_select)
{
    bool dropping_line; /*!< Are we dropping the current line? */
}
PUBLIC_STATE_T(line_select);

IMPORT(line_select);
/*@}*/


#endif /* EOF */

