#ifndef MEMORY_H
#define MEMORY_H
/***************************************************************************
 *   Copyright (C) 2004, 2006 Symbian Software Ltd.                        *
 *   All rights reserved.                                                  *
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@strudl.org                *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/

#include "opts.h"
#include <stddef.h>
#include <stdlib.h>

/*!\ingroup memory_component memory_interface
 *\file memory.h
 * This file provides the Memory component interface
*/

/*!	\addtogroup memory_interface*/
/*@{*/

/*! Type to represent non-const strings allocated on the heap*/
typedef char * heap_str;

/*! Type to represent const strings allocated on the heap*/
typedef char const * heap_str_const;

/*! \c malloc() wrapper; bails on failure*/
extern void *
allocate(size_t bytes);

/*! Allocate a zero-initialised heap block; bails on failure*/
void *
zallocate(size_t bytes);

/*! \c realloc() wrapper; bails on failure*/
extern void *
reallocate(void *ptr, size_t bytes);

/*! \c calloc() wrapper; bails on failure*/
extern void *
callocate(size_t items, size_t size);

/*! Safely free a pointer.
	\param		pp	Address of the pointer to be freed

	If \e *pp is NULL nothing is done. Otherwise \e *pp
	is freed and set = NULL.
*/
extern void
release(void **pp);

/*!	Copy a region of memory into a heap block.
	\param	src	Address of the region to copy.
	\param	len	Length of the region to copy in bytes, or 0. If 0
			then the 0-terminated array of characters at \e src
			is copied.
	\return Pointer to a heap block containing the copied bytes. If
			\e src is NULL then NULL is returned.

	The returned heap block has always a terminal 0.
*/
extern void *
clone(void const *src, size_t len);

/*@}*/

#endif /* EOF*/
